require 'spec_helper'
require 'support/shared_missing_admin'

describe DashOverlord::UseCases::V1::ChartConfigs::Update::Base do
  def make_the_call(id, params = {})
    @context = described_class.perform \
      chart_config_id: id, chart_config_attributes: (params || {}), session: admin_session

    raise_context_unknown_error @context

    @response = app_response @context
  end

  def chart_config_attributes(*args)
    attributes = attributes_for(*args).slice \
      :id,
      :source,
      :function,
      :project_id

    attributes[:source] = attributes[:source].to_s

    attributes
  end

  include_examples 'authenticated admin use_case'

  context 'when current_admin exists' do
    before { @current_admin = create('v1/admin', :streetbees) }

    context 'when sending valid params' do
      before do
        @chart_config = create('v1/chart_config', :pie_chart)

        make_the_call @chart_config.id,
                      chart_config_attributes('v1/chart_config', :percentage)
      end

      it 'response.http_status should return 200' do
        expect(@response.http_status).to be 200
      end

      it 'response.data should return ChartConfig details' do
        existing_chart_config = DashOverlord::Models::V1::ChartConfig.last

        attributes = chart_config_attributes('v1/chart_config', :percentage)

        expect(@response.data).to match a_hash_including({
          'code' => 200,
          'status' => 'ok',
          'data' => a_hash_including(
            { id: existing_chart_config.id }.merge(attributes)
          )
        })
      end
    end

    context 'when requesting an non existing ChartConfig' do
      before do
        make_the_call -1
      end

      it 'response.http_status should return 404' do
        expect(@response.http_status).to be 404
      end

      it 'response.data should return a 404 message' do
        expect(@response.data).to match a_hash_including({
          'code' => 404,
          'message' => I18n.t('v1.chart_configs.not_found'),
          'status' => 'not_found'
        })
      end
    end

    context 'when sending invalid params' do
      before do
        @chart_config = create('v1/chart_config', :pie_chart)

        make_the_call @chart_config.id, source: '', project_id: '', function: ''
      end

      it 'response.http_status should return 422' do
        expect(@response.http_status).to be 422
      end

      it 'response.data should return ChartConfig details' do
        attributes = chart_config_attributes('v1/chart_config', :percentage)

        expect(@response.data).to match a_hash_including({
          'code' => 422,
          'status' => 'unprocessable_entity',
          'data' => a_hash_including({
            'errors' => a_hash_including(
              'source' => [I18n.t('errors.messages.blank')],
              'function' => [I18n.t('errors.messages.invalid')],
              'project_id' => [I18n.t('errors.messages.blank')]
            )
          })
        })
      end
    end
  end
end
