require 'spec_helper'
require 'support/shared_missing_admin'

describe DashOverlord::UseCases::V1::Clients::Update::Base do
  def make_the_call(id, params = {})
    @context = described_class.perform \
      client_id: id, client_attributes: (params || {}), session: admin_session

    raise_context_unknown_error @context

    @response = app_response @context
  end

  def client_attributes(*args)
    attributes = attributes_for(*args).slice \
      :id,
      :uid,
      :name

    attributes
  end

  include_examples 'authenticated admin use_case'

  context 'when current_admin exists' do
    before { @current_admin = create('v1/admin', :streetbees) }

    context 'when sending valid params' do
      before do
        @client = create('v1/client', :unilever)

        make_the_call @client.id, client_attributes('v1/client', :vodafone)
      end

      it 'response.http_status should return 200' do
        expect(@response.http_status).to be 200
      end

      it 'response.data should return Client details' do
        existing_client = DashOverlord::Models::V1::Client.last

        attributes = client_attributes('v1/client', :vodafone)

        expect(@response.data).to match a_hash_including({
          'code' => 200,
          'status' => 'ok',
          'data' => { id: existing_client.id }.merge(attributes)
        })
      end
    end

    context 'when requesting an non existing Client' do
      before do
        make_the_call -1
      end

      it 'response.http_status should return 404' do
        expect(@response.http_status).to be 404
      end

      it 'response.data should return a 404 message' do
        expect(@response.data).to match a_hash_including({
          'code' => 404,
          'message' => I18n.t('v1.clients.not_found'),
          'status' => 'not_found'
        })
      end
    end

    context 'when sending invalid params' do
      before do
        @client = create('v1/client', :unilever)

        make_the_call @client.id, name: ''
      end

      it 'response.http_status should return 422' do
        expect(@response.http_status).to be 422
      end

      it 'response.data should return Client details' do
        attributes = client_attributes('v1/client', :vodafone)

        expect(@response.data).to match a_hash_including({
          'code' => 422,
          'status' => 'unprocessable_entity',
          'data' => a_hash_including({
            'errors' => a_hash_including(
              'name' => [I18n.t('errors.messages.blank')]
            )
          })
        })
      end
    end

    context 'when trying to overwrite the uid' do
      before do
        @client = create('v1/client', :unilever)

        make_the_call @client.id, name: 'Vodafone', uid: 'unilever'
      end

      it 'existing Client must belong to current_admin' do
        client = DashOverlord::Models::V1::Client.last

        attributes = client_attributes('v1/client', :vodafone)

        expect(client.uid).to eq attributes[:uid]
      end
    end
  end
end
