require 'spec_helper'

describe Elasticord::Base, elasticsearch: true do
  let(:index_namespace) { Elasticord::Config.get(:index_namespace) }

  describe '.index' do
    it 'described_class should return index_namespace' do
      expect(described_class.index).to eq index_namespace
    end

    context 'given a child class' do
      let(:child_class) { Class.new(described_class) }

      it 'child_class should return the index_namespace' do
        expect(child_class.index).to eq index_namespace
      end

      context 'given that the child class sets a new index' do
        before do
          child_class.index = 'child_class_new_index'
        end

        it 'described_class should return index_namespace' do
          expect(described_class.index).to eq index_namespace
        end

        it 'child_class should return "child_class_new_index"' do
          expect(child_class.index).to eq 'child_class_new_index'
        end
      end
    end
  end

  describe '.type' do
    it 'should return "_all"' do
      expect(described_class.type).to eq '_all'
    end

    context 'given a child class with a name' do
      let(:child_class) do
        Class.new(described_class) do
          def self.name
            'ChildClass'
          end
        end
      end

      it 'should return that child class name' do
        expect(child_class.type).to eq 'child_class'
      end
    end
  end

  describe '.client' do
    context 'without params' do
      before do
        @client = described_class.client
      end

      it 'should reaturn an instance of Client::Base' do
        expect(@client).to be_an_instance_of Elasticord::Client::Base
      end

      it 'reaturned client should have a protected #entity set up' do
        expect(@client.send(:entity)).to eq described_class
      end

      it 'reaturned client should have a #index set up' do
        expect(@client.index).to eq index_namespace
      end

      it 'reaturned client should have a #type set up' do
        expect(@client.type).to eq '_all'
      end
    end

    context 'sending "project_1" as an argument' do
      before do
        @client = described_class.client 'project_1'
      end

      it 'should reaturn an instance of Client::Base' do
        expect(@client).to be_an_instance_of Elasticord::Client::Base
      end

      it 'reaturned client should have a protected #entity set up' do
        expect(@client.send(:entity)).to eq described_class
      end

      it 'reaturned client should have a protected #index set up' do
        expect(@client.send(:index)).to eq "#{index_namespace}_project_1"
      end

      it 'reaturned client should have a protected #type set up' do
        expect(@client.send(:type)).to eq '_all'
      end
    end

    context 'sending "project_5", "client_2" as arguments' do
      before do
        @client = described_class.client 'project_5', 'client_2'
      end

      it 'should reaturn an instance of Client::Base' do
        expect(@client).to be_an_instance_of Elasticord::Client::Base
      end

      it 'reaturned client should have a protected #entity set up' do
        expect(@client.send(:entity)).to eq described_class
      end

      it 'reaturned client should have a protected #index set up' do
        expect(@client.send(:index)).to eq "#{index_namespace}_project_5"
      end

      it 'reaturned client should have a protected #type set up' do
        expect(@client.send(:type)).to eq '_all_client_2'
      end
    end
  end

  describe '.search' do
    it 'should pass all params to a Client::Base instance' do
      attributes = [{ name: 'test' }]

      expect_any_instance_of(Elasticord::Client::Base).to \
        receive(:search).with(*attributes)

      described_class.search(*attributes)
    end
  end

  describe '.get' do
    it 'should pass all params to a Client::Base instance' do
      attributes = [1]

      expect_any_instance_of(Elasticord::Client::Base).to \
        receive(:get).with(*attributes)

      described_class.get(*attributes)
    end
  end

  describe '.none' do
    it 'should pass all params to a Client::Base instance' do
      expect_any_instance_of(Elasticord::Client::Base).to receive(:none)

      described_class.none
    end
  end

  describe '.create' do
    it 'should pass all params to a Client::Base instance' do
      attributes = [{ name: 'test' }]

      expect_any_instance_of(Elasticord::Client::Base).to \
        receive(:create).with(*attributes)

      described_class.create(*attributes)
    end
  end

  describe '.delete_all' do
    it 'should pass all params to a Client::Base instance' do
      expect_any_instance_of(Elasticord::Client::Base).to \
        receive(:delete_all)

      described_class.delete_all
    end
  end

  describe '.refresh_index' do
    it 'should pass all params to a Client::Base instance' do
      expect_any_instance_of(Elasticord::Client::Base).to \
        receive(:refresh_index)

      described_class.refresh_index
    end
  end

  describe '.delete_by_query' do
    it 'should pass all params to a Client::Base instance' do
      attributes = [{ name: 'test' }]

      expect_any_instance_of(Elasticord::Client::Base).to \
        receive(:delete_by_query).with(*attributes)

      described_class.delete_by_query(*attributes)
    end
  end
end
