require 'spec_helper'

describe Elasticord::Client::Base, elasticsearch: true do
  def get_one_elastic_search_record(id)
    Elasticord.elastic_search_client.get \
      id: id, type: 'child_class', index: ENV['ENV'], ignore: 404
  end

  let(:described_instance) do
    described_class.new \
      Elasticord.elastic_search_client,
      OpenStruct,
      ENV['ENV'],
      'child_class'
  end

  describe '#create' do
    context 'given an ID' do
      let(:id) { '123' }

      before do
        @child_class = described_instance.create id: id, title: 'child class'
      end

      it 'ElasticSearch should contain a record with that specific id' do
        result = get_one_elastic_search_record id

        expect(result['found']).to be true
      end

      it '@child_class.id should match' do
        expect(@child_class.id).to eq id
      end

      it '@child_class should be an instance of OpenStruct' do
        expect(@child_class).to be_an_instance_of OpenStruct
      end
    end

    context 'given no ID' do
      before do
        @child_class = described_instance.create title: 'child class'
      end

      it 'ElasticSearch DB should contain a record with that specific id' do
        result = get_one_elastic_search_record @child_class.id

        expect(result['found']).to be true
      end
    end
  end

  describe '#search' do
    it 'should pass all params to a SearchBuilder instance' do
      attributes = { query: { match: { brand: 'oficial' } } }

      expect_any_instance_of(Elasticord::Client::SearchBuilder).to \
        receive(:load).with(attributes)

      described_instance.search(attributes)
    end
  end

  describe '#get' do
    context 'when the record does not exist' do
      it 'should return false' do
        result = described_instance.get '-1'

        expect(result).to be false
      end
    end

    context 'when the record exists' do
      before do
        @child_class = described_instance.create title: 'child class'

        @result = described_instance.get @child_class.id
      end

      it 'should return an instance of OpenStruct' do
        expect(@result).to be_an_instance_of OpenStruct
      end

      it 'should return the same ID it requested' do
        expect(@result.id).to be @child_class.id
      end
    end
  end

  describe '#none' do
    before do
      @search_builder = described_instance.none
    end

    it 'should reaturn an instance of Client::SearchBuilder' do
      expect(@search_builder).to \
        be_an_instance_of Elasticord::Client::SearchBuilder
    end

    it 'reaturned search_builder should have a protected #entity set up' do
      expect(@search_builder.send(:entity)).to eq OpenStruct
    end

    it 'reaturned search_builder should have a #index set up' do
      expect(@search_builder.index).to eq ENV['ENV']
    end

    it 'reaturned search_builder should have a #type set up' do
      expect(@search_builder.type).to eq 'child_class'
    end
  end
end
