require 'spec_helper'

describe Elasticord::Client::SearchBuilder, elasticsearch: true do
  let(:client_base) do
    Elasticord::Client::Base.new \
      Elasticord.elastic_search_client,
      OpenStruct,
      ENV['ENV'],
      'child_class'
  end

  let(:described_instance) { client_base.none }

  describe '#current_page' do
    it 'should have a default value set' do
      expect(described_instance.current_page).to \
        be described_class::PAGE_DEFAULT
    end
  end

  describe '#per_page' do
    it 'should have a default value set' do
      expect(described_instance.per_page).to \
        be described_class::PER_PAGE_DEFAULT
    end
  end

  describe '#body_params' do
    it 'should have pagination params set' do
      expect(described_instance.body_params).to match a_hash_including({
        size: described_class::PER_PAGE_DEFAULT,
        from: 0
      })
    end

    context 'when setting #page with 2' do
      before do
        @search_builder = described_instance
        @search_builder.page(2)
      end

      context 'when setting #per with 3' do
        before do
          @search_builder.per(3)
        end

        it 'should have pagination params set' do
          expect(@search_builder.body_params).to match a_hash_including({
            size: 3,
            from: 3
          })
        end
      end
    end
  end

  describe '#page' do
    context 'sending 3 as an argument' do
      before do
        @result = described_instance.page(3)
      end

      it 'should return a SearchBuilder instance' do
        expect(@result).to be_an_instance_of described_class
      end

      it 'should set the #body_params used in #load' do
        expect(@result.body_params[:from]).to be 20
      end

      it 'should set the #current_page to 3' do
        expect(@result.current_page).to be 3
      end
    end
  end

  describe '#per' do
    context 'sending 15 as an argument' do
      before do
        @result = described_instance.per(15)
      end

      it 'should return a SearchBuilder instance' do
        expect(@result).to be_an_instance_of described_class
      end

      it 'should set the #body_params used in #load' do
        expect(@result.body_params[:size]).to be 15
      end

      it 'should set the #per_page to 15' do
        expect(@result.per_page).to be 15
      end
    end
  end

  context 'given that no record has been created' do
    before do
      client_base.delete_all
      client_base.refresh_index
    end

    describe '#load' do
      before do
        @search_result = described_instance.load
      end

      it 'should return a pagination object with empty resources' do
        expect(@search_result).to be_empty
        expect(@search_result.total_results).to be 0
      end
    end
  end

  context 'given that 5 records have been created' do
    before do
      client_base.delete_all
      5.times { client_base.create(title: 'child class')  }
      client_base.refresh_index
    end

    describe '#load' do
      before do
        @search_result = described_instance.load
      end

      it 'should find 5 total_results' do
        expect(@search_result.total_results).to be 5
      end

      it 'should return a pagination object with 5 instances of the class' do
        expect(@search_result.length).to be 5

        @search_result.each do |child_class|
          expect(child_class).to be_an_instance_of OpenStruct
        end
      end
    end
  end

  context 'given that 35 records have been created' do
    before do
      client_base.delete_all

      24.times do
        client_base.create \
          title: 'child class', brand: 'oficial', locked: false
      end

      client_base.create(title: 'class', brand: 'oficial', locked: true)

      5.times do
        client_base.create(title: 'child', brand: 'unoficial', locked: false)
      end

      5.times do
        client_base.create(title: 'snow', brand: 'fake', locked: false)
      end

      client_base.refresh_index
    end

    describe '#load' do
      context 'when no pagination params are passed' do
        before do
          @search_result = described_instance.load({})
        end

        it 'should find 35 total_results' do
          expect(@search_result.total_results).to be 35
        end

        it 'should return a pagination object with 10 instances of the class' do
          expect(@search_result.length).to be 10

          @search_result.each do |child_class|
            expect(child_class).to be_an_instance_of OpenStruct
          end
        end
      end

      context 'when #page(2) was called' do
        before do
          @search_builder = described_instance.page(2)
        end

        context 'when #per(15) was called' do
          before do
            @search_builder = @search_builder.per(15)
          end

          context 'when #ransack with multiple query params was called' do
            before do
              @search_builder = @search_builder.ransack({
                brand_eq_any: ['oficial', 'unoficial'],
                locked_eq: false
              })

              @search_result = @search_builder.load
            end

            it 'should return a pagination with the current data' do
              expect(@search_result.per_page).to be 15
              expect(@search_result.total_pages).to be 2
              expect(@search_result.current_page).to be 2
              expect(@search_result.total_results).to be 29
              expect(@search_result.length).to be 14
            end
          end
        end
      end

      context 'when single query params are passed' do
        before do
          @search_result = described_instance.load \
            query: {
              match: { brand: 'oficial' }
            }
        end

        it 'should find 25 total_results' do
          expect(@search_result.total_results).to be 25
        end

        it 'should return a pagination object with 10 resources' do
          expect(@search_result.length).to be 10
        end
      end

      context 'when search for "uniq" and "any" values' do
        before do
          @search_result = described_instance.load \
            query: {
              bool: {
                must: [
                  { term: { locked: false } }
                ],
                should: [
                  { term: { brand: 'oficial' } },
                  { term: { brand: 'unoficial' } }
                ],
                minimum_should_match: 1,
                boost: 1.0
              }
            }
        end

        it 'should find 29 total_results' do
          expect(@search_result.total_results).to be 29
        end

        it 'should return a pagination object with 10 resources' do
          expect(@search_result.length).to be 10
        end
      end
    end
  end
end
